#!/usr/bin/python

from math import *
import numpy as np
import Manyo as m
import Levmar
import time

####    linear combination of gaussian    ####
#def dist(x, n, a, c, w):
def dist(x, param):
    sum=param[0]
    for i in range(len(param[1])):
        a, c, w=param[1][i]
        sum = sum + a*exp(-1.0*((x-c)/w)**2)
    return sum

####    sign function    ####
def sign(r):
    if r > 0.0:
        s = 1.0
    elif r == 0.0:
        s = 0.0
    else:
        s = -1.0
    return s

####    initialize an ElementContainer    ####
#def initElementContainer(xmin, xmax, nDiv):
def initElementContainer(xmin, xmax, nDiv, param):

    bin = m.MakeDoubleVector(nDiv+1)
    delta = (xmax - xmin)/nDiv
    for i in range(nDiv+1):
        bin[i] = xmin + delta*i

    xc = m.MakeDoubleVector(nDiv)
    for i in range(nDiv):
        xc[i] = (bin[i] + bin[i+1])/2.0

    #nPeak=1
    #a=m.MakeDoubleVector(nPeak)
    #c=m.MakeDoubleVector(nPeak)
    #w=m.MakeDoubleVector(nPeak)
    #a[0]=100000.0
    #c[0]=7.5
    #w[0]=2.5
    y = m.MakeDoubleVector(nDiv)
    e = m.MakeDoubleVector(nDiv)
    for i in range(nDiv):
        #p = dist(xc[i], nPeak, a, c, w)
        p = dist(xc[i], param)
        q = sign(np.random.rand(1)-0.5)*(p/10.0)*sqrt(-1.0*log(np.random.rand(1)))
        y[i] = p + q
        e[i] = p/10.0

    ec = m.ElementContainer()
    ec.AddToHeader("run number", 1)
    ec.AddToHeader("level",      1)
    ec.AddToHeader("Inst.",      "manyo")
    ec.Add("TOF",       bin, "sec." )
    ec.Add("Intensity", y,   "count")
    ec.Add("Error",     e,   "count")
    ec.SetKeys("TOF", "Intensity", "Error")

    return ec

def outputElementContainer(ec):
    bin = ec.Put(ec.PutXKey())
    y   = ec.Put(ec.PutYKey())
    e   = ec.Put(ec.PutEKey())

    print str("No.").rjust(4), str("----------- bin -----------").rjust(27), str("xc").rjust(10), str("Intensity").rjust(23), str("error").rjust(23)
    for i in range(ec.PutSize(ec.PutYKey())):
         print str(i).rjust(4), "[", str(bin[i]).rjust(10), ",", str(bin[i+1]).rjust(10), ")", str((bin[i]+bin[i+1])/2.0).rjust(10), str(y[i]).rjust(23), str(e[i]).rjust(23)

####    plot    ####
def plot(src, dest):
    srcBin  = src.Put(src.PutXKey())
    srcY    = src.Put(src.PutYKey())
    destBin = dest.Put(dest.PutXKey())
    destY   = dest.Put(dest.PutYKey())

    count=0
    g=m.GnuplotInterface()
    g.e("set xrange[0: 15]")
    g.e("set yrange[0: 150000]")
    g.e("plot '-' using 1:2 with lines linewidth 1 title'%d', '-' using 1:3 with lines linewidth 4" %count)
    while 1:
        for i in range(srcY.size()):
            #print str( (srcBin[i]+srcBin[i+1])/2.0 ) + " " + str( srcY[i] )+ " " + str( destY[i] )
            g.e( str( (srcBin[i]+srcBin[i+1])/2.0 ) + " " + str( srcY[i] )+ " " + str( destY[i] ) )
        g.e("e")
        c=raw_input("")
        if c != "":
            break

####    main routine    ####
xmin = 0.0
xmax = 15.0
nDiv = 1500
param1=[1000.0,     [[100000.0, 5.0, 1.5], [100000.0, 10.0, 1.5], [35000.0, 2.8, 1.0] ]]

src = initElementContainer(xmin, xmax, nDiv, param1)
outputElementContainer(src)

domain = Levmar.Domain()
domain.setSource(src)
domain.setRange(xmin, xmax)
print "[", str(domain.getLowerBound()).rjust(10), ",", str(domain.getUpperBound()).rjust(10), "]"
print "[", str(domain.getLowerBoundID()).rjust(10), ",", str(domain.getUpperBoundID()).rjust(10), "]"
print
print "type of the domain", domain.getType()
#domain.setType(Domain.OO)
#print "type of the domain", domain.getType()

levmar=Levmar.Levmar()

paramSet = levmar.setDefaultParam(src);
paramSet.dump()

# initial values of fitting parameters
param=m.DoubleVector()
param.push_back(60683.0)
param.push_back(3.42)
param.push_back(1.14)
param.push_back(102955.0)
param.push_back(5.02)
param.push_back(1.66)
param.push_back(100978.0)
param.push_back(10.0)
param.push_back(1.27)

# box constrain for fitting parameters
lb=m.DoubleVector()
for i in range(param.size()):
    lb.push_back(param[i]*0.9)
ub=m.DoubleVector()
for i in range(param.size()):
    ub.push_back(param[i]*1.1)

paramSet.add(Levmar.Levmar.PARAMETER_VALUES, param)
paramSet.add(Levmar.Levmar.LOWER_BOUNDS,     lb)
paramSet.add(Levmar.Levmar.UPPER_BOUNDS,     ub)

#combinations of fitting function
funcStr=""
for i in range(len(param1[1])):
    funcStr=funcStr+" g"
   
parser=Levmar.FuncParser(funcStr) # a string expression for list of function names or symbols
funcList=parser.parse()
paramSet.add(Levmar.Levmar.FUNCTIONS, funcList)

paramSet.replace(Levmar.Levmar.CONSTRAIN,          Levmar.Levmar.NO_CONSTRAIN)  # default Levmar.Levmar.BOX
paramSet.replace(Levmar.Levmar.USE_NUMERICAL_DIFF, False)                             # default True
#paramSet.replace(Levmar.Levmar.DIFF_METHOD,        Levmar.Levmar.CENTRAL)       # default FOWARD
#paramSet.replace(Levmar.Levmar.USE_DATA_WEIGHTS,   False)                             # default True
paramSet.replace(Levmar.Levmar.MAX_ITERATIONS,     1000)                              # default 1000
paramSet.replace(Levmar.Levmar.OUTPUT_INTERVAL,    50)                               # default 50

paramSet.replace(Levmar.Levmar.SCALING_FACTOR,     Levmar.Levmar.DEFAULT_SCALING_FACTOR)     # default 0.001
paramSet.replace(Levmar.Levmar.TOLERANCE,          Levmar.Levmar.DEFAULT_TOLERANCE)          # default 1.0e-17
paramSet.replace(Levmar.Levmar.GRADIENT_TOLERANCE, Levmar.Levmar.DEFAULT_GRADIENT_TOLERANCE) # default 1.0e-17
paramSet.replace(Levmar.Levmar.RELATIVE_TOLERANCE, Levmar.Levmar.DEFAULT_RELATIVE_TOLERANCE) # default 1.0e-17
paramSet.dump()

print "parameter check: ", levmar.checkParam(src, domain, paramSet)  # parameter check, true if parametes are consistent

if  levmar.checkParam(src, domain, paramSet):
    levmar.toInnerForm(src, domain, paramSet)  # translate inner form
    levmar.fit()                               # fitting
    levmar.eval()                              # evaluate values of functions useing fitted parameter
    result=m.ElementContainer()
    levmar.toElementContainer(src, result)     # copy results to a element container
    plot(src, result)
    #outputElementContainer(result)




#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
import os
import Manyo as mm
import Manyo.Utsusemi as mu
import utsusemi.DNA.ana.Reduction.EditWiringFile as ED

import utsusemi.DNA.ana.Reduction.EventDataToEcm as EtoE
import sys,shutil
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
    from elementtree.ElementTree import Element
else:
    from xml.etree.ElementTree import ElementTree
    from xml.etree.ElementTree import Element
try:
    ver_ED = float( ED.__version__ )
    if (ver_ED<0.6):
        raise UserWarning("EditDetectorInfo or EventDataToHist is not valid.")
except:
    raise UserWarning("EditDetectorInfo or EventDataToHist is not valid.")

__version__ = '0.6'

class Histogram(object):
    """
    Load EventData and convert to histogram.
    ver 0.6 [130304] update for Monitor
    ver 0.5 [130213] upgrade
    ver 0.4 [110717] replace EventDataToHist to EventDataToEcm
    ver 0.3 [110104] add control tofDoubleFrameBoundary
    ver 0.2 [101108] can use direct conversion from event data to energy transfer
    ver 0.1 before 101108 original version
    """
    __version__ = '0.5'
    def __init__(self,runNo=-1,
                 ConvType=-1,
                 Params=[],
                 DetRange=[],
                 pixelNo=-1,
                 TimeRange=[],
                 envFile="environ_ana.xml",
                 frameInfo=[0,0.0],
                 DetType="PSD",
                 useTrignet=0,
                 isPulseHeight=False,
                 isDebug=False,
                 t0_offset="0.0",# AS add [t0_offset] 20140522 for DNA
                 tofOffsetFile="",
                 tofSource="0.0,400.0",
                 chopperFrequency=225,
                 isHistogram=True #[inamura 170616]
                ): 
        """
        Constructor and initialization
        
        @param runNo    (int) Run Number 0:Dummy data
        @param ConvType (int) TOF Conversion Type
        @param Params ([])
        @param DetRange ([])  [first PSD ID, final PSD ID]
        @param pixelNo  (int) the number of pixels on one PSD
        @param TimeRange ([]) [startTime, endTime] [seconds]
        @param envFile  (string) Environ file for event data to histogram
        @param frameInfo ([int,double]) FrameNo and Boundary TOF
        @param DetType  (string) PSD or N2MON
        @param isPulseHeight (bool)
        @param isDebug  (bool)
        @param t0_offset(str)
        """

        self.isParallel = False
        self.runNo = runNo
        self.ConvType = ConvType
        
        self.lambda_Center=-1.0 ##[inamura 170612]
        self.lambda_Delta=-1.0  ##[inamura 170612]
        isBadParams = False
        if ((self.ConvType==2) or (self.ConvType==3) or (self.ConvType==12) or (self.ConvType==13) or (self.ConvType==21)):
            if len(Params)<3:
                isBadParams = True
            ##[inamura 170612]-->
            if len(Params)==5:
                self.lambda_Center=Params[3]
                self.lambda_Delta=Params[4]
            ##<--[inamura 170612]
            Params = Params[:3]
        if (self.ConvType==20):
            if len(Params)<5:
                isBadParams = True
            Params = Params[:5]
        if (self.ConvType==22):
            if len(Params)<6:
                isBadParams = True
            Params = Params[:6]
        if isBadParams:
            msg = "Num of Parameters is invalid for type=%d ("
            for p in Params:
                msg += " %f" % (p)
            msg +=")"
            raise UserWarning(msg)

        print("koko94 Params", Params)
        self.Params = Params
        self.startDet = DetRange[0]
        self.endDet = DetRange[1]
        self.numOfPixelOnDet = pixelNo
        if len(TimeRange)>=2:
            self.SliceEventStart = TimeRange[0]
            self.SliceEventEnd = TimeRange[1]
        else:
            self.SliceEventStart = -1.0
            self.SliceEventEnd = -1.0
        self.envFile = ""
        if envFile.find(",")==-1:
            self.envFile = envFile
        else:
            self.envFile = "environ_ana.xml"

        self.TOFOffsetFile = tofOffsetFile

        self.frameInfo = frameInfo
        self.DetType = DetType
        self.useTrignet = useTrignet
        self.isPulseHeight = isPulseHeight
        self.isHistogram = isHistogram
        self.isDebug = isDebug
        
        ##self.L1 = EtoE.Def_L1*1000.0 #[mm]
        self.L1 = -1.0
        self.T0shift = 0.0
        #self.instCode = os.environ["UTSUSEMI_SYS_NAME"]
        self.instCode = mu.UtsusemiEnvGetInstCode()
        self.CommentHead = "Histogram >>>"
        
        self.DetectorInfo = None
        ana_env = mu.UtsusemiAnaEnvironReader(self.envFile)
        self.param_files={"WiringInfo":"","DetectorInfo":""}
        if not ana_env._Status:
            print(self.CommentHead+"environ file ("+self.envFile+") is not existed.")
            self.envFile = ""
            self.anaEnv = None
            self.Status = -1
            return
        else:
            self.anaEnv = ana_env
            
            param_files = self.anaEnv.PutParamFiles(self.runNo)
            if param_files.size()<2:
                msg = "Run No is not suitable to choose WiringInfo from environ_ana.xml"
                raise UserWarning(msg)

            ind = envFile.find(",")
            if self.isDebug:
                print("---================ Check envFile=",envFile,"ind=",ind)
            if ind==-1:
                self.param_files["WiringInfo"]=param_files[0]
                self.param_files["DetectorInfo"]=param_files[1]
            else:
                if self.isDebug:
                    print("=================== Changed by envFile")
                self.param_files["WiringInfo"]=envFile[:ind]
                if envFile[(ind+1):]!="":
                    self.param_files["DetectorInfo"]=envFile[(ind+1):]
                else:
                    self.param_files["DetectorInfo"]=param_files[1]
                
            self.Status = 0
            #self.DetectorInfo = mu.UtsusemiDetectorInfoReader3HePsd( self.anaEnv.Info["PositioningFile"] )
            path_dfile = ""
            #if os.path.exists( self.anaEnv.Info["PositioningFile"] ):
            #    path_dfile = self.anaEnv.Info["PositioningFile"]
            #else:
            #    path_dfile = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",self.anaEnv.Info["PositioningFile"])
            #    if not os.path.exists( path_dfile ):
            #        self.Status = -1
            #        print "Cannot find DetectorInfo file "+self.anaEnv.Info["PositioningFile"]
            #        return

            # 20151202 start
            fname = self.param_files["DetectorInfo"][:-4]
            footer= self.param_files["DetectorInfo"][-4:]
            pid = str(os.getpid())

            if os.path.exists( fname+"_"+pid+footer ):
                path_dfile = fname+"_"+pid+footer
            else:
                #path_dfile = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",fname+"_"+pid+footer)
                path_dfile = os.path.join(mu.UtsusemiEnvGetUserDir(), "ana", "xml",fname+"_"+pid+footer)
                if not os.path.exists( path_dfile ):
                    self.Status = -1
                    print("Cannot find DetectorInfo file "+self.param_files["DetectorInfo"])
                    return
            # 20151202 end
                
            #self.DetectorInfo = mu.UtsusemiDetectorInfoReader3HePsd( path_dfile )
            self.DetectorInfo = mu.DetectorInfoEditorNeunet( path_dfile )
            self.L1 = self.DetectorInfo.PutInstL1()

        if useTrignet>0:
            self.caseInfoFile     = "cfileFiltDNA.xml"

            #20151202 start
            #chopCondInfoOrgFile = "cfileChopDNA.xml"
            chopCondInfoOrgFile = "cfileChopDNA170612.xml" ##[inamura 170612]
            #cpath  = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",chopCondInfoOrgFile)
            cpath  = os.path.join(mu.UtsusemiEnvGetBaseDir(),self.instCode,"ana","xml",chopCondInfoOrgFile)
            fname  = chopCondInfoOrgFile[:-4]
            footer = chopCondInfoOrgFile[-4:]
            pid = str(os.getpid())

            self.chopCondInfoFile=fname+"_"+pid+footer
            #self.chopCondInfoFile = "cfileChopDNA.xml"
            #CCpath = os.path.join(os.environ["HOME"],"ana","xml",self.chopCondInfoFile)
            #CCpath=os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",self.chopCondInfoFile)
            CCpath = os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","xml",self.chopCondInfoFile)
            shutil.copy(cpath,CCpath)
            #20151202 end
            #----------------------------------------------------------
            def _getHeader(dpath):
                headStr=""
                if dpath!="":
                    head=open(dpath)
                    l=head.readline()
                    if l[:2]=="<?":headStr+=l
                    l=head.readline()
                    if l[:2]=="<!":headStr+=l

                return headStr
            #----------------------------------------------------------
            def _appendHeader(dpath,headStr):
                allline=open(dpath,"r").read()

                f = open(dpath,"w")
                f.write(headStr)
                for line in allline:f.write(line)
                f.close()
            #----------------------------------------------------------
            def _writeParam(val,dpath,params,att="",i=0):
                if dpath!="":
                    headStr=_getHeader(dpath)

                    fd = open(dpath,'rb')
                    dom=ElementTree(file=fd)
                    fd.close()

                    ele = dom.getroot()
                    for param in params:
                        if param==params[-1]:
                            ele = ele.findall(param)
                        else:
                            ele = ele.find(param)
                    valStr=str(val)

                    if att=="":
                        ele[i].text=valStr
                    else:
                        ele[i].set(att,valStr)
                    dom.write(dpath)
                    _appendHeader(dpath,headStr)
            #----------------------------------------------------------
            def _selectSlit(dpath,useTrignet,chopperFrequency=225):
                if dpath!="":
                    headStr=_getHeader(dpath)

                    fd = open(dpath,'rb')
                    dom=ElementTree(file=fd)
                    fd.close()

                    ele = dom.getroot()

                    for e in ele.find("fastChopper").findall("slit"):
                        ele.find("fastChopper").remove(e)

                    slit3cm = zip(["0","1","2"],["90","180","270"],["5.73","5.73","5.73"])               ,"3"
                    slit1cm = zip(["0"],        ["0"],             ["1.91"])                             ,"1"
                    slitAll = zip(["0","1","2","3"],["0","90","180","270"],["1.91","5.73","5.73","5.73"]),"4"

                    if useTrignet==1 or useTrignet==4:
                        slitTags=slitAll
                    elif useTrignet==2 or useTrignet==5:
                        slitTags=slit1cm
                    elif useTrignet==3 or useTrignet==6:
                        slitTags=slit3cm

                    ele.find("fastChopper").set("n",slitTags[1])
                    ele.find("fastChopper").set("frequency",str(chopperFrequency))
                    for i,angle,width in slitTags[0]:
                        elem=Element("slit")
                        elem.set("i",i)
                        elem.set("angle",angle)
                        elem.set("width",width)
                        ele.find("fastChopper").append(elem)

                    dom.write(dpath)
                    _appendHeader(dpath,headStr)
            #----------------------------------------------------------

            ##[inamura 170612]-->
            _writeParam(str(self.lambda_Center),CCpath,["lambdaCenter"])
            _writeParam(str(self.lambda_Delta),CCpath,["lambdaDelta"])
            ##<--[inamura 170612]

            if useTrignet==1 or useTrignet==2 or useTrignet==3: # Use Trignet

                _selectSlit(CCpath,useTrignet,chopperFrequency)

                if "," in t0_offset:
                    offsets = t0_offset.split(",")
                elif  "/" in t0_offset:
                    offsets = t0_offset.split("/")
                else:
                    offsets = [t0_offset]
                try:
                    if len(offsets)==0:
                        tof_offset    = 0
                        tof_resolver1 = 0
                        tof_resolver2 = 0
                    elif len(offsets)==1:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = 0
                        tof_resolver2 = 0
                    elif len(offsets)==2:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = float(offsets[1])
                        tof_resolver2 = 0
                    elif len(offsets)>=3:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = float(offsets[1])
                        tof_resolver2 = float(offsets[2])
                except:
                    tof_offset    = 0
                    tof_resolver1 = 0
                    tof_resolver2 = 0

                _writeParam(str(tof_offset),CCpath,["tofOffset"])

                if DetType=="uPSD":
                    _writeParam(str(-tof_resolver1)+","+str(-tof_resolver2), CCpath,["tofResolver"])
                else:
                    _writeParam(str(-tof_resolver1)+","+str(-tof_resolver2), CCpath,["tofResolver"])

                if "/" in tofSource:
                    tofSource = ",".join(tofSource.split("/"))
                _writeParam(tofSource, CCpath,["tofSource"])

            elif useTrignet==4 or useTrignet==5 or useTrignet==6: # Not Use Trignet
                _selectSlit(CCpath,useTrignet,chopperFrequency)

                if "," in t0_offset:
                    offsets = t0_offset.split(",")
                elif  "/" in t0_offset:
                    offsets = t0_offset.split("/")
                else:
                    offsets = [t0_offset]
                try:
                    if len(offsets)==0:
                        tof_offset    = 0
                        tof_resolver1 = 0
                        tof_resolver2 = 0
                    elif len(offsets)==1:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = 0
                        tof_resolver2 = 0
                    elif len(offsets)==2:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = float(offsets[1])
                        tof_resolver2 = 0
                    elif len(offsets)>=3:
                        tof_offset    = float(offsets[0])
                        tof_resolver1 = float(offsets[1])
                        tof_resolver2 = float(offsets[2])
                except:
                    tof_offset    = 0
                    tof_resolver1 = 0
                    tof_resolver2 = 0

                _writeParam(str(tof_offset),CCpath,["tofOffset"])

                if DetType=="uPSD":
                    _writeParam(str(-tof_resolver1)+","+str(-tof_resolver2), CCpath,["tofResolver"])
                else:
                    _writeParam(str(-tof_resolver1)+","+str(-tof_resolver2), CCpath,["tofResolver"])

                if "/" in tofSource:
                    tofSource = ",".join(tofSource.split("/"))
                _writeParam(tofSource, CCpath,["tofSource"])

            self.t0_offset = 0  # AS add [t0_offset] 20140522 for DNA
        else:
            try:
                if "," in t0_offset:
                    self.t0_offset = float(t0_offset.split(",")[0])
                elif  "/" in t0_offset:
                    self.t0_offset = float(t0_offset.split("/")[0])
                else:
                    self.t0_offset = float(t0_offset)
            except:
                self.t0_offset = 0

            self.caseInfoFile     = ""
            ##[inamura 180911]-->
            #self.chopCondInfoFile = ""
            self.chopCondInfoFile = "%f,%f"%(self.lambda_Center,self.lambda_Delta)
            print("### [inamura 180911] chopCondInfoFile="+self.chopCondInfoFile)
            ##<--[inamura 180911]

    def MakeWiringFile(self,WIoutput=None):
        if self.anaEnv== None:
            print(self.CommentHead+"Set file name of environment file (loadEnvFile).")
            return
        
        #path_org_wf = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",self.param_files["WiringInfo"])
        path_org_wf = os.path.join(mu.UtsusemiEnvGetBaseDir(),self.instCode,"ana","xml",self.param_files["WiringInfo"])
        if not os.path.exists(path_org_wf):
            print(self.CommentHead+"ERROR: There is no original wiring file,",path_org_wf)
            return -1
        
        return self._MakeWiringFile(path_org_wf,WIoutput=WIoutput)
            

    def _MakeWiringFile(self,path_org_wf,p=None,WIoutput=None):
        print(self.CommentHead+"Read Original WiringInfo ("+path_org_wf+")")
        ed = ED.EditWiringInfo(path_org_wf)
        daq_list=ed.InstInfo.PutContainerIdList()

        for daq_r in daq_list:
            ed.SuspendDaq(daq_r)
        
        if (self.startDet>=0) and (self.endDet<=ed.MaxDetId):
            for i in range(int(self.startDet),int(self.endDet+1)):
                (dam,daq_i,mod_i) = ed.DetInfoList[i]
                if self.isDebug:
                    print("dam,daq_i,mod_i",dam,daq_i,mod_i)
                for j in ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainerIdList():
                    info = ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainer(j) 
                    info.SetStatus(1)
        elif self.startDet<0 and self.endDet<0:
            for daq_r in daq_list:
                ed.ResumeDaq(daq_r)
        elif self.startDet<0 and self.endDet<=ed.MaxDetId:
            for daq_r in daq_list:
                ed.ResumeDaq(daq_r)
            for i in range( int(self.endDet),int(ed.MaxDetId+1)):
                for j in ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainerIdList():
                    info = ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainer(j) 
                    info.SetStatus(0)
        elif self.startDet>=0 and self.endDet<0:
            for i in range( int(self.startDet),int(ed.MaxDetId+1)):
                for j in ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainerIdList():
                    info = ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainer(j) 
                    info.SetStatus(1)
        ed.SelectByDetType( self.DetType )
        
        if self.frameInfo[0]!=0:
            ed.set_FrameInfo( self.frameInfo[0],self.frameInfo[1] )
            if self.isDebug:
                print(self.CommentHead+("Set FrameNo %d Boundary TOF %f" % (self.frameInfo[0],self.frameInfo[1] )))
        
        if self.numOfPixelOnDet!=-1:
            ed.ReSetNumOfPixel( 0,ed.MaxDetId, 0, self.numOfPixelOnDet )
            if self.isDebug:
                print(self.CommentHead+"Change the number of pixels on one PSD to ",self.numOfPixelOnDet)

        if self.ConvType!=-1:
            ptnId = ed.change_TofBinPattern(type=self.ConvType,cont_in=self.Params)
            #ed.setall_TofBinInfo(ptnId,0)
            ed.setall_TofBinInfo(ptnId,self.t0_offset) # AS add [t0_offset] 20140522 for DNA
            print(self.CommentHead+"ConvType=",self.ConvType,", Params = ",self.Params)
        
        self.param_files["WiringInfo"] = "%s_%3s%06d_%06d_%s" % (os.environ["USER"],self.instCode,self.runNo,os.getpid(),self.param_files["WiringInfo"])

        #if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
        if WIoutput is None:
            udir = mu.UtsusemiEnvGetUserDir()
            if udir!="":
                ss = os.path.join(udir,"ana","xml",self.param_files["WiringInfo"])
            else:
                #ss = os.path.join(os.environ["HOME"],"ana","xml",self.param_files["WiringInfo"])
                ss = os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","xml",self.param_files["WiringInfo"])
        else:
            ss = WIoutput
        ed.SaveToXml( ss )
        comm = "chmod go+w "+ss
        print(self.CommentHead+"Do command = ",comm)
        try:
            os.system(comm)
        except:
            print(self.CommentHead+" Command missed.")
        #------------------------------------------------------------------------------------------------------------

        import sys
        ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
        if ver<2.5:
            from elementtree.ElementTree import ElementTree,SubElement
        else:
            from xml.etree.ElementTree import ElementTree,SubElement

        def _writeParam(dpath,items1,items2):
            if dpath!="":
                headStr=""
                head=open(dpath)
                l=head.readline()
                if l[:2]=="<?":headStr+=l
                l=head.readline()
                if l[:2]=="<!":headStr+=l

                fd = open(dpath,'rb')
                dom=ElementTree(file=fd)
                fd.close()

                ele = dom.getroot()

                ele=ele.find("pixelInfo")
                num=len(ele.findall("daq"))

                sele=SubElement(ele,"daq")
                ele.set("n",str(num+1))
                for item in items1:
                    if item[0]=="i":
                        sele.set(item[0],str(num))
                    else:
                        sele.set(item[0],item[1])

                sele2=SubElement(sele,"module")
                for item in items2:
                    sele2.set(item[0],item[1])

                dom.write(dpath)

                allline=open(dpath,"r").read()

                f = open(dpath,"w")
                f.write(headStr)
                for line in allline:f.write(line)
                f.close()
        #------------------------------------------------------------------------------------------------------------
        def _readParam(dpath,params):
            if dpath!="":
                fd = open(dpath,'rb')
                dom=ElementTree(file=fd)
                fd.close()

                ele = dom.getroot()
                for param,j in params:
                    if j!=0:
                        ele = ele.findall(param)[j]
                    else:
                        ele = ele.find(param)
                    if ele==None:
                        print("input param is not found",param)
                        return ""

                return ele.items()

        def _copyParamTrignt():
            items1 = _readParam(path_org_wf,[("pixelInfo",0),("daq",3)])
            items2 = _readParam(path_org_wf,[("pixelInfo",0),("daq",3),("module",0)])
            _writeParam(ss,items1,items2)
        try:
            _copyParamTrignt()
        except:
            print("_copyParamTrignt Error")
    def GetHistogram(self,daq=-1):
        if self.anaEnv==None:
            print(self.CommentHead+" There is no information.")
            return -1

        #20151202 start
        fname  = self.param_files["DetectorInfo"][:-4]
        footer = self.param_files["DetectorInfo"][-4:]
        pid = str(os.getpid())
        #20151202 end
        DAT = EtoE.GetHistogram(eventtype=self.DetType,
                                runnumber=self.runNo,
                                WiringInfo_xml=self.param_files["WiringInfo"],
                                DetectorInfo_xml=fname+"_"+pid+footer,
                                CaseInfo_xml=self.caseInfoFile,
                                ChopCondInfo_xml=self.chopCondInfoFile,
                                TOFOffsetFile=self.TOFOffsetFile,
                                useTrignet=self.useTrignet,
                                starttime=self.SliceEventStart,
                                endtime=self.SliceEventEnd,
                                isPH=self.isPulseHeight,
                                isHistogram=self.isHistogram )#[inamura 170616]
        """
        DAT = EtoE.GetHistogram( self.DetType, self.runNo,
                                 p.Info["wiringfile"],
                                 self.anaEnv.Info["PositioningFile"],
                                 self.SliceEventStart,self.SliceEventEnd )
        """
        """
        if not self.isDebug:
            comm = "rm -rf %s" % (os.path.join(os.environ["HOME"],"ana","xml",self.param_files["WiringInfo"]))
            try:
                os.system(comm)
            except:
                print self.CommentHead+" False to delete %s."%(os.path.join(os.environ["HOME"],"ana","xml",self.param_files["WiringInfo"]))
        """
        return DAT


###############################################
### Sample for utilization of Histogram class
###############################################
def GetHistDTconst(runNo=-1,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0,detType="PSD"):
    """
    Load EventData and convert to histogram.
    GetHistogram(runNo=-1,paraFlag=0,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0)
    @param runNo    (int) Run Number
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param bTOF     (double) boundary tof for double frame mode
    @param detType  (string) PSD or N2MON
    @retval ElementContainerMatrix
    """
    fInfo = [0,0.0]
    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [-1.0, -1.0]
    h=Historam(runNo=runNo,
               ConvType=2,
               Params=params,
               DetRange=drange,
               pixelNo=pixelNo,
               TimeRange=trange,
               frameInfo=fInfo,
               DetType=detType,
               )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetHistogramSimple( runNo=-1,wiringFile="WiringFile_org.xml",detectCompFile="DetectorPosi2.xml", startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0 ):
    """
    Load EventData and confert to histogram using given WiringFile and DetectorCompFile.
    GetHistogramSimple( runNo=-1,wiringFile="WiringFile_org.xml",detectCompFile="DetectorPosi2.xml", 
                        startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0
                      )
    @param runNo    (int) Run Number
    @param wiringFile (string) Wiring File name
    @param detectCompFile (string) Detector Components file name
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param bTOF (double) boundary tof for double frame mode
    @retval ElementContainerMatrix
    """
    in_envFile = wiringFile+","+detectCompFile
    in_envFile = in_envFile.replace(" ","")

    fInfo = [0,0.0]

    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [-1.0, -1.0]
    h=Histogram(runNo=runNo,
                ConvType=-1,
                Params=params,
                DetRange=drange,
                pixelNo=pixelNo,
                TimeRange=trange,
                envFile=in_envFile,
                frameInfo=fInfo,
                DetType="PSD",
                isPulseHeight=False,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetHistogramByTime(runNo=-1,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,startTime=-1.0,endTime=-1.0,bTOF=0.0):
    """
    Load EventData and convert to histogram.
    Histogram(runNo,paraFlag,startTOF=-1,endTOF=-1,binTOF=-1,pixelNo=-1,startPSD=-1,endPSD=-1,envFile)
    @param runNo    (int) Run Number
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param startTime (float)
    @param endTime   (float)
    @param bTOF (double) boundary tof for double frame mode
    @retval ElementContainerMatrix
    """
    fInfo = [0,0.0]
    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [startTime, endTime]
    h=Histogram(runNo=runNo,
                ConvType=2,
                Params=params,
                DetRange=drange,
                pixelNo=pixelNo,
                TimeRange=trange,
                frameInfo=fInfo,
                DetType="PSD",
                isPulseHeight=False,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()
    
def GetHistogramOfConv( runNo=-1,
                        ConvType=-1,
                        Params=[],
                        detectorRange=[-1,-1],
                        numOfPixel=-1,
                        timeRange=[-1.0,-1.0],
                        frameInfo=[0,0.0],
                        DetType="PSD",
                        useTrignet=0,
                        t0_offset="0.0", # AS add [t0_offset] 20140522 for DNA
                        tofOffsetFile = "",
                        tofSource="0.0,400.0",
                        ChopperFrequency=225.0,
                        isHistogram=True,
                        WIoutput = None
                        ):
                        
    """
    Load EventData and convert to histogram
    GetHistogramOfConv( runNo=-1,
                        ConvType=-1,
                        Params=[],
                        detectorRange=[-1,-1],
                        numOfPixel=-1,
                        timeRange=[-1.0,-1.0],
                        frameInfo=[0,0.0],
                        DetType="PSD"
                        )
    @param runNo         (int)   run number
    @param ConvType      (int)   Conversion Type 
    @param Params        (list)  parameters required by ConvType
    @param detectorRange (list)  range of detectors [startId, endId]
    @param numOfPixel    (int)   the number of pixels of a PSD
    @param timeRange     (list)  time range [start Time[sec], end Time[sec] ]
    @param frameInfo     (list)  frame info
    @param DetType       (string) "PSD" or "N2MON"
    @retval ElementContainerMatrix
    """
    h=Histogram(runNo=runNo,
                ConvType=ConvType,
                Params=Params,
                DetRange=detectorRange,
                pixelNo=numOfPixel,
                TimeRange=timeRange,
                frameInfo=frameInfo,
                DetType=DetType,
                useTrignet=useTrignet,
                isPulseHeight=False,
                isDebug=False,
                t0_offset=t0_offset,# AS add [t0_offset] 20140522 for DNA
                tofOffsetFile=tofOffsetFile,
                tofSource=tofSource,
                chopperFrequency=ChopperFrequency,
                isHistogram=isHistogram #[inamura 170616]
                )

    if h.Status==0:
        if WIoutput is not None:
            h.MakeWiringFile(WIoutput=WIoutput)
            return None
        h.MakeWiringFile()
        return h.GetHistogram()

def GetPulseHeight(runNo=-1,
                   detType="PSD",
                   phHist=[0,10000,10],
                   detectorRange=[-1,-1],
                   timeRange=[-1.0,-1.0]
                   ):
    
    """
    Load EventData and make PulseHeight hist
    GetPulseHeight(runNo=-1,
                   detType="PSD",
                   phHist=[0,10000,10],
                   detectorRange=[-1,-1],
                   timeRange=[-1.0,-1.0]
                   )
    @param runNo
    @param detType
    @param phHist
    @param detectorRange
    @param timeRange
    @retval ElementContainerMatrix
    """
    h=Histogram(runNo=runNo,
                ConvType=2,
                Params=phHist,
                DetRange=detectorRange,
                pixelNo=1,
                TimeRange=timeRange,
                DetType=detType,
                isPulseHeight=True,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetAnalysisParameter(laticeSpace):

    Params=[0.0   , 1.0,    0.1,      0.0,      0.0,     0.0,       laticeSpace, 0.0]
    #      [Hw_min, Hw_max, Hw_width, Lambda_c, dLambda, t0_offset, LaticeSpace, L1]
    """
    h=Histogram(runNo=-1,
                ConvType=27,
                Params=Params,
                DetRange=[-1,-1],
                DetType="uPSD"
                )

    if h.Status==0:
        h.MakeWiringFile()

    WiringInfo_xml   = h.anaEnv.Info["SingleMode"].Info["wiringfile"]
    DetectorInfo_xml = h.anaEnv.Info["PositioningFile"]

    if os.environ.has_key("UTSUSEMI_USR_PRIV_HOME"):
        WIpath=os.path.join(os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","xml",WiringInfo_xml)
    else:
        WIpath=os.path.join(os.environ["HOME"],"ana","xml",WiringInfo_xml)
    """

    #[inamura 180907]-->
    ana_env = mu.UtsusemiAnaEnvironReader("environ_ana.xml")
    param_files = ana_env.PutParamFiles(1)
    #wfile = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",param_files[0])
    wfile = os.path.join(mu.UtsusemiEnvGetBaseDir(),mu.UtsusemiEnvGetInstCode(),"ana","xml",param_files[0])
    ed = ED.EditWiringInfo(wfile)
    ed.SelectByDetType( "uPSD" )
    wfile_tmp = "%s_DNA000000_%06d_%s" % (os.environ["USER"],os.getpid(),param_files[0])
    #WIpath = os.path.join( os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","xml",wfile_tmp )
    WIpath = os.path.join( mu.UtsusemiEnvGetUserDir(),"ana","xml",wfile_tmp )
    ed.SaveToXml(WIpath)
    DetectorInfo_xml = param_files[1]
    #<--[inamura 180907]
    #DCpath=os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",DetectorInfo_xml)
    DCpath=os.path.join(mu.UtsusemiEnvGetBaseDir(),mu.UtsusemiEnvGetInstCode(),"ana","xml",DetectorInfo_xml)

    import Manyo.DNA as md

    evt = md.UtsusemiEventDataConverterDNA()
    evt.LoadParamFiles( WIpath, DCpath )
    evt.SetHistAllocation()

    ef = evt.PutEf()
    pa = evt.PutPolarAngle()
    aa = evt.PutAzimuthAngle()

    ed = ED.EditWiringInfo(WIpath)
    sum_of_det,sum_of_pixel=ed.PutNumOfBoth()

    efArray=[]
    aaArray=[]
    paArray=[]
    i=0
    for detId in range(sum_of_det):
        (flag,daq,mod) = ed.DetInfoList[detId]
        info = ed.InstInfo.PutContainer(daq).PutContainer(mod).PutContainer(detId)
        did, numOfPixel, numAxis= info.PutPixelInfo()
        efTmp=[]
        aaTmp=[]
        paTmp=[]
        for j in range(numOfPixel):
            efTmp.append(ef[i])
            aaTmp.append(aa[i])
            paTmp.append(pa[i])
            i+=1
        efArray.append(efTmp)
        aaArray.append(aaTmp)
        paArray.append(paTmp)

    del evt

    #[inamura 180907]-->
    #import cPickle as pickle
    #fo = open("/home/dnaadmin/YI_180907_new_ef.dmp","w")
    #pickle.dump( efArray, fo )
    #fo.close()
    #fo = open("/home/dnaadmin/YI_180907_new_aa.dmp","w")
    #pickle.dump( aaArray, fo )
    #fo.close()
    #fo = open("/home/dnaadmin/YI_180907_new_pa.dmp","w")
    #pickle.dump( paArray, fo )
    #fo.close()
    ##sys.exit(0)
    #<--[inamura 180907]

    return efArray,aaArray,paArray




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ElasticScanGUIQt.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDoubleSpinBox, QGroupBox,
    QLabel, QMainWindow, QMenuBar, QPushButton,
    QRadioButton, QSizePolicy, QSpinBox, QStatusBar,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(452, 719)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.errorstr = QLabel(self.centralwidget)
        self.errorstr.setObjectName(u"errorstr")
        self.errorstr.setGeometry(QRect(100, 0, 241, 31))
        font = QFont()
        font.setPointSize(14)
        font.setBold(True)
        self.errorstr.setFont(font)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(10, 20, 431, 241))
        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 20, 67, 31))
        self.run = QSpinBox(self.groupBox)
        self.run.setObjectName(u"run")
        self.run.setGeometry(QRect(70, 22, 91, 26))
        self.run.setMaximum(999999)
        self.label_3 = QLabel(self.groupBox)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(170, 20, 101, 31))
        self.ChMenu = QComboBox(self.groupBox)
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.addItem("")
        self.ChMenu.setObjectName(u"ChMenu")
        self.ChMenu.setGeometry(QRect(270, 22, 51, 25))
        self.readtemp_bt = QPushButton(self.groupBox)
        self.readtemp_bt.setObjectName(u"readtemp_bt")
        self.readtemp_bt.setGeometry(QRect(330, 22, 89, 25))
        self.groupBox_2 = QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.groupBox_2.setGeometry(QRect(10, 50, 411, 61))
        self.label_4 = QLabel(self.groupBox_2)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(0, 30, 51, 21))
        self.label_5 = QLabel(self.groupBox_2)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(120, 30, 71, 21))
        self.Sta = QDoubleSpinBox(self.groupBox_2)
        self.Sta.setObjectName(u"Sta")
        self.Sta.setGeometry(QRect(40, 25, 81, 26))
        self.Sta.setDecimals(1)
        self.Sta.setMaximum(9999999.000000000000000)
        self.End = QDoubleSpinBox(self.groupBox_2)
        self.End.setObjectName(u"End")
        self.End.setGeometry(QRect(180, 25, 81, 26))
        self.End.setDecimals(1)
        self.End.setMaximum(9999999.000000000000000)
        self.End.setValue(180.000000000000000)
        self.label_6 = QLabel(self.groupBox_2)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(260, 30, 101, 21))
        self.Bin = QDoubleSpinBox(self.groupBox_2)
        self.Bin.setObjectName(u"Bin")
        self.Bin.setGeometry(QRect(350, 25, 51, 26))
        self.Bin.setDecimals(1)
        self.Bin.setMaximum(9999999.000000000000000)
        self.Bin.setValue(60.000000000000000)
        self.groupBox_3 = QGroupBox(self.groupBox)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.groupBox_3.setGeometry(QRect(10, 110, 411, 61))
        self.label_7 = QLabel(self.groupBox_3)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(0, 30, 51, 21))
        self.label_8 = QLabel(self.groupBox_3)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setGeometry(QRect(120, 30, 21, 21))
        self.label_9 = QLabel(self.groupBox_3)
        self.label_9.setObjectName(u"label_9")
        self.label_9.setGeometry(QRect(300, 30, 51, 21))
        self.label_10 = QLabel(self.groupBox_3)
        self.label_10.setObjectName(u"label_10")
        self.label_10.setGeometry(QRect(150, 30, 51, 21))
        self.label_11 = QLabel(self.groupBox_3)
        self.label_11.setObjectName(u"label_11")
        self.label_11.setGeometry(QRect(270, 30, 21, 21))
        self.Ti = QDoubleSpinBox(self.groupBox_3)
        self.Ti.setObjectName(u"Ti")
        self.Ti.setGeometry(QRect(40, 25, 81, 26))
        self.Ti.setDecimals(1)
        self.Ti.setMaximum(9999999.000000000000000)
        self.Tf = QDoubleSpinBox(self.groupBox_3)
        self.Tf.setObjectName(u"Tf")
        self.Tf.setGeometry(QRect(180, 25, 81, 26))
        self.Tf.setDecimals(1)
        self.Tf.setMaximum(9999999.000000000000000)
        self.Tf.setValue(300.000000000000000)
        self.TBin = QDoubleSpinBox(self.groupBox_3)
        self.TBin.setObjectName(u"TBin")
        self.TBin.setGeometry(QRect(330, 25, 51, 26))
        self.TBin.setDecimals(1)
        self.TBin.setMaximum(9999999.000000000000000)
        self.TBin.setValue(1.000000000000000)
        self.groupBox_4 = QGroupBox(self.groupBox)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.groupBox_4.setGeometry(QRect(10, 179, 411, 51))
        self.rb1_NFinished = QRadioButton(self.groupBox_4)
        self.rb1_NFinished.setObjectName(u"rb1_NFinished")
        self.rb1_NFinished.setGeometry(QRect(20, 20, 112, 23))
        self.rb1_Finished = QRadioButton(self.groupBox_4)
        self.rb1_Finished.setObjectName(u"rb1_Finished")
        self.rb1_Finished.setGeometry(QRect(170, 20, 112, 23))
        self.groupBox_5 = QGroupBox(self.centralwidget)
        self.groupBox_5.setObjectName(u"groupBox_5")
        self.groupBox_5.setGeometry(QRect(10, 270, 431, 231))
        self.label_12 = QLabel(self.groupBox_5)
        self.label_12.setObjectName(u"label_12")
        self.label_12.setGeometry(QRect(10, 30, 41, 17))
        self.label_13 = QLabel(self.groupBox_5)
        self.label_13.setObjectName(u"label_13")
        self.label_13.setGeometry(QRect(130, 30, 81, 17))
        self.label_15 = QLabel(self.groupBox_5)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setGeometry(QRect(210, 30, 91, 17))
        self.rb2_FCOFF = QRadioButton(self.groupBox_5)
        self.rb2_FCOFF.setObjectName(u"rb2_FCOFF")
        self.rb2_FCOFF.setGeometry(QRect(30, 50, 381, 23))
        self.rb2_FCON_11_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_11_225.setObjectName(u"rb2_FCON_11_225")
        self.rb2_FCON_11_225.setGeometry(QRect(30, 70, 381, 23))
        self.rb2_FCON_33_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_33_225.setObjectName(u"rb2_FCON_33_225")
        self.rb2_FCON_33_225.setGeometry(QRect(30, 90, 381, 23))
        self.rb2_FCON_1S_225 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_1S_225.setObjectName(u"rb2_FCON_1S_225")
        self.rb2_FCON_1S_225.setGeometry(QRect(30, 110, 381, 23))
        self.rb2_FCON_33_300 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_33_300.setObjectName(u"rb2_FCON_33_300")
        self.rb2_FCON_33_300.setGeometry(QRect(30, 130, 381, 23))
        self.rb2_FCON_11_300 = QRadioButton(self.groupBox_5)
        self.rb2_FCON_11_300.setObjectName(u"rb2_FCON_11_300")
        self.rb2_FCON_11_300.setGeometry(QRect(30, 150, 381, 23))
        self.label_17 = QLabel(self.groupBox_5)
        self.label_17.setObjectName(u"label_17")
        self.label_17.setGeometry(QRect(10, 180, 51, 17))
        self.label_18 = QLabel(self.groupBox_5)
        self.label_18.setObjectName(u"label_18")
        self.label_18.setGeometry(QRect(140, 180, 51, 17))
        self.label_19 = QLabel(self.groupBox_5)
        self.label_19.setObjectName(u"label_19")
        self.label_19.setGeometry(QRect(270, 180, 51, 17))
        self.label_20 = QLabel(self.groupBox_5)
        self.label_20.setObjectName(u"label_20")
        self.label_20.setGeometry(QRect(390, 180, 41, 17))
        self.label_21 = QLabel(self.groupBox_5)
        self.label_21.setObjectName(u"label_21")
        self.label_21.setGeometry(QRect(10, 210, 71, 17))
        self.T0shift = QSpinBox(self.groupBox_5)
        self.T0shift.setObjectName(u"T0shift")
        self.T0shift.setGeometry(QRect(70, 205, 81, 26))
        self.T0shift.setMaximum(400000)
        self.T0shift.setValue(12325)
        self.label_22 = QLabel(self.groupBox_5)
        self.label_22.setObjectName(u"label_22")
        self.label_22.setGeometry(QRect(160, 210, 51, 17))
        self.dqs1 = QDoubleSpinBox(self.groupBox_5)
        self.dqs1.setObjectName(u"dqs1")
        self.dqs1.setGeometry(QRect(50, 25, 81, 26))
        self.dqs1.setDecimals(2)
        self.dqs1.setMinimum(0.010000000000000)
        self.dqs1.setMaximum(9.990000000000000)
        self.dqs1.setSingleStep(0.010000000000000)
        self.dqs1.setValue(0.050000000000000)
        self.emin = QDoubleSpinBox(self.groupBox_5)
        self.emin.setObjectName(u"emin")
        self.emin.setGeometry(QRect(60, 175, 81, 26))
        self.emin.setDecimals(2)
        self.emin.setMinimum(-999.000000000000000)
        self.emin.setMaximum(999.000000000000000)
        self.emin.setSingleStep(1.000000000000000)
        self.emin.setValue(-4.000000000000000)
        self.emax = QDoubleSpinBox(self.groupBox_5)
        self.emax.setObjectName(u"emax")
        self.emax.setGeometry(QRect(190, 175, 81, 26))
        self.emax.setDecimals(2)
        self.emax.setMinimum(-999.000000000000000)
        self.emax.setMaximum(999.000000000000000)
        self.emax.setSingleStep(1.000000000000000)
        self.emax.setValue(4.000000000000000)
        self.de = QDoubleSpinBox(self.groupBox_5)
        self.de.setObjectName(u"de")
        self.de.setGeometry(QRect(310, 175, 81, 26))
        self.de.setDecimals(2)
        self.de.setMinimum(0.010000000000000)
        self.de.setMaximum(99.000000000000000)
        self.de.setSingleStep(0.010000000000000)
        self.de.setValue(2.000000000000000)
        self.groupBox_6 = QGroupBox(self.centralwidget)
        self.groupBox_6.setObjectName(u"groupBox_6")
        self.groupBox_6.setGeometry(QRect(10, 510, 431, 131))
        self.label_23 = QLabel(self.groupBox_6)
        self.label_23.setObjectName(u"label_23")
        self.label_23.setGeometry(QRect(10, 30, 41, 17))
        self.label_14 = QLabel(self.groupBox_6)
        self.label_14.setObjectName(u"label_14")
        self.label_14.setGeometry(QRect(130, 30, 81, 17))
        self.label_16 = QLabel(self.groupBox_6)
        self.label_16.setObjectName(u"label_16")
        self.label_16.setGeometry(QRect(210, 30, 91, 17))
        self.rb3_CHOFF = QRadioButton(self.groupBox_6)
        self.rb3_CHOFF.setObjectName(u"rb3_CHOFF")
        self.rb3_CHOFF.setGeometry(QRect(30, 50, 381, 23))
        self.rb3_CHON_11_225 = QRadioButton(self.groupBox_6)
        self.rb3_CHON_11_225.setObjectName(u"rb3_CHON_11_225")
        self.rb3_CHON_11_225.setGeometry(QRect(30, 70, 381, 23))
        self.label_24 = QLabel(self.groupBox_6)
        self.label_24.setObjectName(u"label_24")
        self.label_24.setGeometry(QRect(390, 100, 41, 20))
        self.label_25 = QLabel(self.groupBox_6)
        self.label_25.setObjectName(u"label_25")
        self.label_25.setGeometry(QRect(270, 100, 51, 17))
        self.label_26 = QLabel(self.groupBox_6)
        self.label_26.setObjectName(u"label_26")
        self.label_26.setGeometry(QRect(10, 100, 51, 17))
        self.label_27 = QLabel(self.groupBox_6)
        self.label_27.setObjectName(u"label_27")
        self.label_27.setGeometry(QRect(140, 100, 51, 17))
        self.dqs3 = QDoubleSpinBox(self.groupBox_6)
        self.dqs3.setObjectName(u"dqs3")
        self.dqs3.setGeometry(QRect(50, 25, 81, 26))
        self.dqs3.setDecimals(2)
        self.dqs3.setMinimum(0.010000000000000)
        self.dqs3.setMaximum(9.990000000000000)
        self.dqs3.setSingleStep(0.010000000000000)
        self.dqs3.setValue(0.050000000000000)
        self.emin3 = QDoubleSpinBox(self.groupBox_6)
        self.emin3.setObjectName(u"emin3")
        self.emin3.setGeometry(QRect(60, 95, 81, 26))
        self.emin3.setDecimals(0)
        self.emin3.setMinimum(-999.000000000000000)
        self.emin3.setMaximum(999.000000000000000)
        self.emin3.setSingleStep(0.010000000000000)
        self.emin3.setValue(-500.000000000000000)
        self.emax3 = QDoubleSpinBox(self.groupBox_6)
        self.emax3.setObjectName(u"emax3")
        self.emax3.setGeometry(QRect(190, 95, 81, 26))
        self.emax3.setDecimals(0)
        self.emax3.setMinimum(0.000000000000000)
        self.emax3.setMaximum(999.000000000000000)
        self.emax3.setSingleStep(4.000000000000000)
        self.emax3.setValue(100.000000000000000)
        self.de3 = QDoubleSpinBox(self.groupBox_6)
        self.de3.setObjectName(u"de3")
        self.de3.setGeometry(QRect(310, 95, 81, 26))
        self.de3.setDecimals(2)
        self.de3.setMinimum(0.010000000000000)
        self.de3.setMaximum(99.000000000000000)
        self.de3.setSingleStep(0.010000000000000)
        self.de3.setValue(50.000000000000000)
        self.button1 = QPushButton(self.centralwidget)
        self.button1.setObjectName(u"button1")
        self.button1.setGeometry(QRect(10, 650, 151, 25))
        self.button3 = QPushButton(self.centralwidget)
        self.button3.setObjectName(u"button3")
        self.button3.setGeometry(QRect(380, 650, 61, 25))
        self.background = QWidget(self.centralwidget)
        self.background.setObjectName(u"background")
        self.background.setGeometry(QRect(0, 0, 441, 691))
        self.background.setStyleSheet(u"")
        MainWindow.setCentralWidget(self.centralwidget)
        self.background.raise_()
        self.errorstr.raise_()
        self.groupBox.raise_()
        self.groupBox_5.raise_()
        self.groupBox_6.raise_()
        self.button1.raise_()
        self.button3.raise_()
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 452, 20))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.run, self.ChMenu)
        QWidget.setTabOrder(self.ChMenu, self.readtemp_bt)
        QWidget.setTabOrder(self.readtemp_bt, self.Sta)
        QWidget.setTabOrder(self.Sta, self.End)
        QWidget.setTabOrder(self.End, self.Bin)
        QWidget.setTabOrder(self.Bin, self.Ti)
        QWidget.setTabOrder(self.Ti, self.Tf)
        QWidget.setTabOrder(self.Tf, self.TBin)
        QWidget.setTabOrder(self.TBin, self.rb1_NFinished)
        QWidget.setTabOrder(self.rb1_NFinished, self.rb1_Finished)
        QWidget.setTabOrder(self.rb1_Finished, self.dqs1)
        QWidget.setTabOrder(self.dqs1, self.rb2_FCOFF)
        QWidget.setTabOrder(self.rb2_FCOFF, self.rb2_FCON_11_225)
        QWidget.setTabOrder(self.rb2_FCON_11_225, self.rb2_FCON_33_225)
        QWidget.setTabOrder(self.rb2_FCON_33_225, self.rb2_FCON_1S_225)
        QWidget.setTabOrder(self.rb2_FCON_1S_225, self.rb2_FCON_33_300)
        QWidget.setTabOrder(self.rb2_FCON_33_300, self.rb2_FCON_11_300)
        QWidget.setTabOrder(self.rb2_FCON_11_300, self.emin)
        QWidget.setTabOrder(self.emin, self.emax)
        QWidget.setTabOrder(self.emax, self.de)
        QWidget.setTabOrder(self.de, self.T0shift)
        QWidget.setTabOrder(self.T0shift, self.dqs3)
        QWidget.setTabOrder(self.dqs3, self.rb3_CHOFF)
        QWidget.setTabOrder(self.rb3_CHOFF, self.rb3_CHON_11_225)
        QWidget.setTabOrder(self.rb3_CHON_11_225, self.emin3)
        QWidget.setTabOrder(self.emin3, self.emax3)
        QWidget.setTabOrder(self.emax3, self.de3)
        QWidget.setTabOrder(self.de3, self.button1)
        QWidget.setTabOrder(self.button1, self.button3)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.errorstr.setText(QCoreApplication.translate("MainWindow", u"Elastic Scan Stand-by.", None))
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Run", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Run No.", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Temp. Sensor", None))
        self.ChMenu.setItemText(0, QCoreApplication.translate("MainWindow", u"A", None))
        self.ChMenu.setItemText(1, QCoreApplication.translate("MainWindow", u"B", None))
        self.ChMenu.setItemText(2, QCoreApplication.translate("MainWindow", u"C", None))
        self.ChMenu.setItemText(3, QCoreApplication.translate("MainWindow", u"C2", None))
        self.ChMenu.setItemText(4, QCoreApplication.translate("MainWindow", u"D", None))
        self.ChMenu.setItemText(5, QCoreApplication.translate("MainWindow", u"D2", None))
        self.ChMenu.setItemText(6, QCoreApplication.translate("MainWindow", u"D3", None))
        self.ChMenu.setItemText(7, QCoreApplication.translate("MainWindow", u"D4", None))
        self.ChMenu.setItemText(8, QCoreApplication.translate("MainWindow", u"D5", None))
        self.ChMenu.setItemText(9, QCoreApplication.translate("MainWindow", u"PEACE_U", None))
        self.ChMenu.setItemText(10, QCoreApplication.translate("MainWindow", u"PEACE_M", None))
        self.ChMenu.setItemText(11, QCoreApplication.translate("MainWindow", u"PEACE_L", None))
        self.ChMenu.setItemText(12, QCoreApplication.translate("MainWindow", u"PEACE_UB", None))
        self.ChMenu.setItemText(13, QCoreApplication.translate("MainWindow", u"PEACE_MB", None))
        self.ChMenu.setItemText(14, QCoreApplication.translate("MainWindow", u"PEACE_LB", None))

        self.readtemp_bt.setText(QCoreApplication.translate("MainWindow", u"Read Temp.", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("MainWindow", u"Time [sec]", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Start:", None))
        self.label_5.setText(QCoreApplication.translate("MainWindow", u"sec. End:", None))
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"sec. Time Bin:", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("MainWindow", u"Temperature", None))
        self.label_7.setText(QCoreApplication.translate("MainWindow", u"Start:", None))
        self.label_8.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_9.setText(QCoreApplication.translate("MainWindow", u"Bin:", None))
        self.label_10.setText(QCoreApplication.translate("MainWindow", u"End:", None))
        self.label_11.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("MainWindow", u"Reduction from event data", None))
        self.rb1_NFinished.setText(QCoreApplication.translate("MainWindow", u"Not Finished", None))
        self.rb1_Finished.setText(QCoreApplication.translate("MainWindow", u"Finished", None))
        self.groupBox_5.setTitle(QCoreApplication.translate("MainWindow", u"Si(111)", None))
        self.label_12.setText(QCoreApplication.translate("MainWindow", u"dQ = ", None))
        self.label_13.setText(QCoreApplication.translate("MainWindow", u"Qmin= 0.0", None))
        self.label_15.setText(QCoreApplication.translate("MainWindow", u"Qmax= 2.0", None))
        self.rb2_FCOFF.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper OFF (Emin=-200,Emax=50,dE=50ueV)", None))
        self.rb2_FCON_11_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,225Hz,Ereso=2.5ueV)", None))
        self.rb2_FCON_33_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (3cm,3cm,225Hz,Ereso=3.5ueV)", None))
        self.rb2_FCON_1S_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,STOP,225Hz,Ereso=3.0ueV)", None))
        self.rb2_FCON_33_300.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (3cm,3cm,300Hz,Ereso=3.1ueV)", None))
        self.rb2_FCON_11_300.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,300Hz,Ereso=2.4ueV)", None))
        self.label_17.setText(QCoreApplication.translate("MainWindow", u"Emin=", None))
        self.label_18.setText(QCoreApplication.translate("MainWindow", u", Emax=", None))
        self.label_19.setText(QCoreApplication.translate("MainWindow", u", dE=", None))
        self.label_20.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.label_21.setText(QCoreApplication.translate("MainWindow", u"T0Shift=", None))
        self.label_22.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.groupBox_6.setTitle(QCoreApplication.translate("MainWindow", u"Si(333)", None))
        self.label_23.setText(QCoreApplication.translate("MainWindow", u"dQ=", None))
        self.label_14.setText(QCoreApplication.translate("MainWindow", u"Qmin= 0.0", None))
        self.label_16.setText(QCoreApplication.translate("MainWindow", u"Qmax= 2.0", None))
        self.rb3_CHOFF.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper OFF (Emin=-500,Emax=100,dE=50ueV)", None))
        self.rb3_CHON_11_225.setText(QCoreApplication.translate("MainWindow", u"Fast Chopper ON (1cm,1cm,225Hz)", None))
        self.label_24.setText(QCoreApplication.translate("MainWindow", u"ueV", None))
        self.label_25.setText(QCoreApplication.translate("MainWindow", u", dE=", None))
        self.label_26.setText(QCoreApplication.translate("MainWindow", u"Emin=", None))
        self.label_27.setText(QCoreApplication.translate("MainWindow", u", Emax=", None))
        self.button1.setText(QCoreApplication.translate("MainWindow", u"Reduction Start", None))
        self.button3.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
    # retranslateUi




# -*- coding: utf-8 -*-
import sys
from pyqtgraph.Qt import QtCore, QtGui, uic
import pyqtgraph as pg
import pyqtgraph.opengl as gl
import numpy as np
import math
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from pyqtgraph.opengl.GLGraphicsItem import GLGraphicsItem

class GLLabelItem(GLGraphicsItem):
    def __init__(self,  antialias=True, glOptions='translucent'):
        GLGraphicsItem.__init__(self)
        self.antialias = antialias
        self.setGLOptions(glOptions)
        self.color=[0, 0, 1, .6]
        self.xpos=0
        self.ypos=0
        self.zpos=0
        self.label="label"
        self.fontType = GLUT_BITMAP_TIMES_ROMAN_24
        pass

    def setColor(self,x):
        self.color=x

    def getColor(self):
        return self.color

    def setPosition(self,x,y,z):
        self.xpos=x
        self.ypos=y
        self.zpos=z

    def getPositon(self):
        return [self.xpos,self.ypos,self.zpos]

    def setText(self,val):
        self.label=val

    def getText(self):
        return self.label

    def setTextSize(self,val):
        self.textSize=val

    def getTextSize(self):
        return self.textSize
    
    def setFontType(self,val):
        self.fontType = val
    
    def paint(self):
        self.setupGLState()
        if self.antialias:
            glEnable(GL_LINE_SMOOTH)
            glHint(GL_LINE_SMOOTH_HINT, GL_NICEST)

        self.drawLabel()

    def drawLabel(self):

        glColor4f(self.color[0],
                  self.color[1],
                  self.color[2],
                  self.color[3])
        glRasterPos3f( self.xpos, self.ypos, self.zpos )
        for c in self.label:
            glutBitmapCharacter( self.fontType, ord(c) )
            #GLUT_BITMAP_8_BY_13
            #GLUT_BITMAP_9_BY_15
            #GLUT_BITMAP_HELVETICA_10
            #GLUT_BITMAP_HELVETICA_12
            #GLUT_BITMAP_HELVETICA_18
            #GLUT_BITMAP_TIMES_ROMAN_10
            #GLUT_BITMAP_TIMES_ROMAN_24


# -*- coding: utf-8 -*-
"""
UtilQt.py
ユーティリティモジュール

History by Y.Inamura
[inamura 170126] changed to send "map-format" data to M2Plot (for VisualContQ)
                 - add a new argument about file name extention to TempFile::GetTempName
[inamura 120512] add marker face status "fill" or "none"
[inamura 110303] add header info for CalcDspacing.
[inamura 110223] save and load default attribute value of FastPlot
"""
from __future__ import print_function
import time
import codecs
import sys
from xml.dom.minidom import parse
import tempfile
import os.path
import os
import platform
if platform.python_version_tuple()[0] == '2':
    import cPickle as pickle
else:
    import pickle


try:
    import Manyo
except:
    pass

try:
    from PySide6 import QtGui, QtWidgets
except:
    try:
        from PySide2 import QtGui, QtWidgets
    except:
        from PySide import QtGui
        import PySide.QtGui as QtWidgets
# リソースファイル名称定義
RESOURCEFILE = "FastVisualize"

# メッセージファイル名称定義
MESSAGEFILE = "Message.xml"
# マスク値定義
# MASKVALUE = 100000000.0
try:
    import Manyo.Utsusemi as mu
    MASKVALUE = mu.UTSUSEMIMASKVALUE64
except:
    MASKVALUE = 1.0E15

#######################################
#  IFEvtProp
#######################################


class IFEvtProp(object):
    """
    イベントとプロパティ授受用のインターフェイスクラス
    """

    dicInstance = {}

    #########################################
    def __new__(cls, idno=0):
        """
        idno 毎に異なるインスタンスを返すコンストラクタ
        (同一グループ間では共通のインスタンスを使用)
        @param  cls 　クラス
        @param  idno  グループID
          　　　　　　プロッタ番号をIDとして使用
        @retval クラスのインスタンス
        """

        # 指定されたID のクラスインスタンスが生成済みなら
        if idno in cls.dicInstance:
            # 生成済みのインスタンスを返す
            instance = cls.dicInstance[idno]
        else:
            # クラスインスタンスが未生成なら新規に構築
            instance = object.__new__(cls)

        return instance

    #########################################
    def __init__(self, idno=0):
        """
        インターフェイスクラスのコンストラクタ
        @param  idno  グループID
        @retval クラスのインスタンス
        """
        # クラスインスタンスが登録済みなら
        if idno in self.dicInstance:
            # 何もしない
            return

        # クラスインスタンスを登録
        self.dicInstance[idno] = self

        # 登録リストを初期化
        self.InitIF()

    #########################################

    def InitIF(self):
        """
        リスナーリストとプロパティディクショナリの初期化
        @param  無し
        @retval 無し
        """
        # リスナーリストの初期化
        self.listner = []
        # プロパティディクショナリの初期化
        self.property = {}

    #########################################
    def AddListner(self, evtType, func):
        """
        イベントに対するリスナー登録追加
        @param  evtType  イベントタイプ(文字列)
        @param  func     イベント発生時に実行する関数
        @retval 無し
        """
        # イベント発生時に実行する関数を登録
        self.listner.append((evtType, func))

    #########################################
    def AddProperty(self, propKey, func):
        """
        公開するプロパティを登録
        @param  propKey  属性名称(文字列)
        @param  func     プロパティが要求されたときに実行する関数(ラムダ式)
        @retval 無し
        """
        # プロパティを返すラムダ式を登録
        self.property[propKey] = func

    #########################################
    def NotifyEvent(self, wid, evtType, value=None):
        """
        イベントに登録されている関数を実行
        (同一イベントに複数登録されている場合は、順番にすべて実行)
        @param  wid      イベント発生元のインスタンス
        @param  evtType  発生したイベントのタイプ(文字列)
        @param  value    イベントの値(省略可)
        @retval 無し
        """
        # リスナーリストから、イベントに登録されているものを探す
        for listner in self.listner:
            evt, func = listner
            # イベントタイプが一致したなら
            if evt == evtType:
                # イベントに登録されている関数を全て実行
                func(*(wid, evtType, value))

    #########################################
    def GetProperty(self, propKey):
        """
        プロパティ取得
        @param  propKey  属性名称(文字列)
        @retval プロパティ
        """
        retval = None
        # プロパティのkey から lambda 式を取得
        try:
            try:
                func = self.property[propKey]
            except:
                raise PlotException(
                    'Common', 'C001', ("Key error:" + propKey, "IFEvtProp:GetProperty"))
            else:
                # プロパティを取得(lambda 式実行)
                try:
                    retval = func()
                except:
                    raise PlotException(
                        'Common', 'C001', ("Lambda execution", "IFEvtProp:GetProperty"))
        except PlotException as ex:
            PlotMessage(None, ex)

        return retval


#######################################
#  MakeTempFile
#######################################
class TempFile(object):
    """
    テンポラリファイルクラス
    """
    ###################################################

    def GetTempName(self, ext=""):  # [inamura 170126]
        """
        テンポラリファイルの名称を取得
        (モジュール tempfile を使用してファイル名を取得すると、
        消せなくなるため、自分でファイル名を作成)
        @param  無し
        @retval テンポラリファイル名称(フルパス)
                 名称取得に失敗した場合は、None を返す
        """
        # 現在の絶対時間(0.01秒単位)と、プロッタ番号よりテンポラリファイル名を作成
        tsec = int(time.time() * 100.0)
        # tmpf = "tmp%d" % tsec
        tmpf = "tmp%d%s" % (tsec, ext)  # [inamura 170126]
        # テンポラリファイルの絶対パスを作成(指定されたテンポラリディレクトリの下)
        tmppath = os.path.join(tempfile.gettempdir(), tmpf)

        # テンポラリファイルの存在チェック
        for i in range(100):
            # 万一同じファイル名があったら、ファイル名を変更
            if os.path.exists(tmppath):
                # tmppath = "%s%d" % (tmppath, i)
                tmppath = "%s%d%s" % (tmppath, i, ext)  # [inamura 170126]
            else:
                # 無ければOK
                break
            if i == 99:
                return None

        return tmppath

    ###################################################
    def MakeTempFile(self, object, temppath):
        """
        Python オブジェクトをシリアライズして
        テンポラリファイルに書き込む
        @param  object    シリアライズするオブジェクト
        @param  temppath  シリアライズしたオブジェクトを書きこむファイル
        @retval  0: 成功、1: テンポラリファイルオープン失敗、2: シリアライズ失敗
        """
        try:
            # テンポラリファイルを開く
            fw = open(temppath, "wb")
        except:
            return 1
        try:
            # オブジェクトをバイナリモードでシリアライズ
            pickle.dump(object, fw, True)
        except:
            # テンポラリファイルをクローズ
            fw.close()
            return 2
        # テンポラリファイルをクローズ
        fw.close()
        # 正常終了
        return 0

    ###################################################
    def ReadTempFile(self, temppath):
        """
        テンポラリファイルを読み込み、デシリアライズかる。
        @param  temppath  シリアライズしたオブジェクトを書きこんだファイル
        @retval  デシリアライズしたデータ、デシリアライズ失敗時は None
        """
        try:
            # テンポラリファイルを読み込む
            fr = open(temppath, "rb")
            # データを復元(デシリアライズ)
            data = pickle.load(fr)
        except:
            return None
        # テンポラリファイルをクローズ
        fr.close()
        # テンポラリファイルを削除
        os.remove(temppath)
        # デシリアライズしたオブジェクトを返す
        return data

#######################################
#  AcqPath
#######################################


class AcqPath(object):
    """
    ディレクトリパス取得ユーティリティ
    """
    ###########################################################################

    def GetHome(self):
        """
        ユーザホームディレクトリ取得
        @param    無し
        @retval   ディレクトリのパス (string)
        """
        # ユーザのホームディレクトリを取得
        try:
            home = os.environ['UTSUSEMI_USR_DIR']
        except:
            # Windows 上で環境変数を設定していない場合はカレントディレクトリとする
            home = os.getcwd()

        return home

    ###########################################################################
    def GetHomeExp(self):
        """
        ユーザホームディレクトリ下の exp パスを取得
        @param    無し
        @retval   ディレクトリのパス (string)
        """
        # ユーザのホームディレクトリを取得
        home = self.GetHome()
        return os.path.join(home, 'exp')

    ###########################################################################
    def GetPrmPath(self):
        """
        ユーザパラメータのパスを取得
        格納場所はホームディレクトリ下の　exp/xml
        @param    無し
        @retval   パラメータディレクトリのパス
        """
        # ホーム下のexpを取得
        exp = self.GetHomeExp()
        return os.path.join(exp, 'xml')

    ###########################################################################
    def GetCurrentMth(self):
        """
        カレントメソッドのパスを取得
        @param    無し
        @retval   カレントファイルのパス (string)
        """
        # ユーザのホームディレクトリを取得
        exp = self.GetHomeExp()
        return os.path.join(exp, CURRENT_FILE)

    ###########################################################################
    def GetSysExp(self):
        """
        システムディレクトリ下のexpパスを取得
        格納場所はファイルサーバ
        @param    無し
        @retval   システムディレクトリ下のexpパス
        """
        # システムディレクトリを取得
        try:
            sysd = os.environ["UTSUSEMI_BASE_DIR"]
        except:
            if os.name == "nt":
                sysd = 'c:\\mlf'
            else:
                sysd = '/usr/local/mlf'

        return os.path.join(sysd, 'exp')

    ###########################################################################
    def GetSysAnal(self):
        """
        システムディレクトリ下のanalパスを取得
        格納場所はファイルサーバ
        @param    無し
        @retval   システムディレクトリ下の anal パス
        """
        # システムディレクトリを取得
        try:
            sysd = os.environ["UTSUSEMI_BASE_DIR"]
        except:
            if os.name == "nt":
                sysd = 'c:\\mlf'

            else:
                sysd = '/usr/local/mlf'

        return os.path.join(sysd, 'ana')

    ###########################################################################
    def GetSysVis(self):
        """
        システムディレクトリ下のvisパスを取得
        格納場所はファイルサーバ
        @param    無し
        @retval   システムディレクトリ下の vis パス
        """
        # システムディレクトリを取得
        try:
            sysd = os.environ["UTSUSEMI_BASE_DIR"]
        except:
            if os.name == "nt":
                sysd = 'c:\\mlf'

            else:
                sysd = '/usr/local/mlf'

        return os.path.join(sysd, 'vis')

    ###########################################################################
    def GetDefaultMth(self):
        """
        デフォルトメソッドファイルのパス取得
        ベースディレクトリ下の　exp/xml
        @param    無し
        @retval   ディレクトリのパス (string)
        """
        # ベースディレクトリを取得
        base = os.environ["UTSUSEMI_BASE_DIR"]
        if base is None:
            if os.name == "nt":
                base = 'c:\\mlf'
            else:
                base = '/usr/local/mlf'

        sysName = os.environ["UTSUSEMI_INST_CODE"]
        if sysName is None:
            sysName = "AMR"

        path = os.path.join(base, sysName)
        path = os.path.join(path, 'exp')
        path = os.path.join(path, 'xml')
        return os.path.join(path, DEFAULT_FILE)

    ###########################################################################
    def GetPmtPath(self, flag):
        """
        PMTファイルのパス取得
        ベースディレクトリ下の　ana/xml
        @param    flag  測定フラグ
        @retval   ディレクトリのパス (string)
        """
        # ベースディレクトリを取得
        try:
            # base = os.environ["UTSUSEMI_USR_DIR"]  ##[okazaki, inamura 140310]
            base = os.environ.get("UTSUSEMI_USR_DIR", None) \
                or mu.UtsusemiGetInstDir()
            #       or os.enviton["UTSUSEMI_USR_DIR"]
        except:
            if os.name == "nt":
                base = 'c:\\mlf'
            else:
                base = '/usr/local/mlf'
        if flag:
            path = os.path.join(base, 'exp')
        else:
            path = os.path.join(base, 'ana')

        # ディレクトリがあるか
        # [okazaki, inamura 140310]-->
        # if not os.path.isdir(path):
        #    os.mkdir(path)
        # PMT用ディレクトリがあるか
        # ppath = os.path.join(path,'pmt')
        # if not os.path.isdir(ppath):
        #    os.mkdir(ppath)
        # return ppath
        try:
            if not os.path.isdir(path):
                os.mkdir(path)
            # PMT用ディレクトリがあるか
            pmtpath = os.path.join(path, 'pmt')
            if not os.path.isdir(pmtpath):
                os.mkdir(pmtpath)
        except:
            # pmtpath = os.environ["HOME"]
            pmtpath = os.getcwd()
        return pmtpath
        # <--[okazaki, inamura 140310]

    #########################################

    def GetModulePath(self, module):
        """
        モジュールパスの探索
        @param  module  　ファイル名
        @retval モジュールのフルパス
        """
        # カレントディレクトリを取得
        path = os.getcwd()
        fpath = os.path.join(path, module)
        # カレントディレクトリに目的ファイルがあるか
        if os.path.isfile(fpath):
            # フルパスを返す
            return fpath

        # 環境変数に指定されたディレクトリを取得
        paths = [self.GetSysExp(), self.GetSysAnal(), self.GetSysVis()]
        # 全PYTHONPATHディレクトリを探索
        for path in paths:
            fpath = os.path.normpath(os.path.join(path, module))
            # ディレクトリに目的ファイルがあるか
            if os.path.isfile(fpath):
                # フルパスを返す
                return fpath

        # 目的ファイルが見つからなかった
        return None


#######################################
#  PlotException
#######################################
class PlotException(Exception):
    """
    例外処理用コンテナクラス
    測定条件決定支援処理プログラム内で例外オブジェクトとして使用する
    メッセージ及びエラーコードは当クラスから取得できる
    """

    def __init__(self, parent, key, tap):
        """
        コンストラクタ
        @param parent:メッセージのカテゴリ
        @param key:メッセージID
        @param tap:編集パラメータ(tupleオブジェクト）

        """
        self.parent = parent
        self.key = key
        self.tap = tap

    ################################
    def GetMessage(self):
        """
        メッセージの取得
        @params 無し
        @retval msg:メッセージ
        """
        msg = MessageFile()
        return msg.GetMessage(self.parent, self.key, self.tap)

    ################################
    def GetErrorCode(self):
        """
        エラーコードの取得
        @param 無し
        @retval code:エラーコード
        """
        return self.code

    ################################
    def GetDetailCode(self):
        """
        詳細コードの取得
        @param 無し
        @retval detail:詳細コード
        """
        return self.detail

#######################################
#  MessageFile
#######################################


class MessageFile(object):
    """
    MessageFileクラス
    MessageFileの読み込みとMessageの編集を行う
    Singletonクラスとして利用
    """
    instance = None
    lang = 'English'

    ################################
    def __new__(cls, *args):
        """
        Singletonコンストラクタ
        @param 無し
        @retval　無し
        """
        if cls.instance is None:
            cls.instance = object.__new__(cls, *args)
            # メッセージファイル読み込み
            cls.instance.Load()
        return cls.instance

    ################################
    def Load(self):
        """
        MessageFileの読み込み
        @param 　無し
        @retval 無し
        """
        # メッセージファイルの絶対パスを作る
        msgpath = self.SearchPath(MESSAGEFILE)
        try:
            # メッセージファイルをXML パーサに読み込み
            self.xmlDoc = parse(msgpath)
            self.isRead = True
        except:
            self.isRead = False

    ################################
    def GetMessage(self, parent, key, tap):
        """
        編集されたMessageの取得
        @param parent:メッセージのカテゴリ
        @param key:メッセージID
        @param tap:編集パラメータ(tupleオブジェクト）
        @retval Message
        """
        # --> 2008/04/28 Minakawa
        # メッセージファイルを読み込み済みか
        if not self.isRead:
            self.Load()
        # <-- 2008/04/28 Minakawa

        # メッセージファイルの読み込みに成功しているか
        if self.isRead:
            # 親ノードを取得
            node = self.xmlDoc.getElementsByTagName(parent)[0]
            # 言語ノードを取得
            lang = node.getElementsByTagName(self.lang)[0]
            try:
                # メッセージ文字列を取得
                child = lang.getElementsByTagName(key)[0]
            except TypeError:
                raise PlotException('Common', 'C001', (key, "Wrong Code"))
            try:
                # 文字列にタプルで指定された引数を入れこむ
                return child.firstChild.data % (tap)
            except TypeError:
                raise PlotException('Common', 'C001', (key, "TypeError"))
        else:
            return 'Message File Read Error!! ( Message.GetMessage() )'

    #########################################
    def SearchPath(self, fname):
        """
        フルパスの探索
        @param  fname  　ファイル名
        @retval ファイルのフルパス
        """
        # カレントディレクトリを取得
        self.filePath = os.getcwd()
        fpath = os.path.join(self.filePath, fname)
        # カレントディレクトリに目的ファイルがあるか
        if os.path.isfile(fpath):
            # フルパスを返す
            return fpath
        self.filePath = ""
        # 環境変数 UTSUSEMI_BASE_DIR に指定されたディレクトリを取得
        try:
            path = os.environ["UTSUSEMI_BASE_DIR"]
            path = os.path.join(path, "vis")
            fpath = os.path.join(path, fname)
            if os.path.isfile(fpath):
                # フルパスを返す
                return fpath

        except:
            return ""

        # [inamura 100720]-->
        # for StandAlone
        if path is None:
            p_path = os.environ.get('PYTHONPATH')
            if p_path is not None:
                p_path_list = p_path.split(os.pathsep)
                path_list = []
                for pp in p_path_list:
                    path_list.append(pp)
                    path_list.append(os.path.join(pp, "vis"))
                for pp in path_list:
                    fp = os.path.normpath(os.path.join(pp, fname))
                    if os.path.isfile(fp):
                        self.filePath = pp
                        return fp
        else:
            return ""

        """
        path = os.path.join(path,'vis')
        fpath = os.path.join(path,fname)

        # ディレクトリに目的ファイルがあるか
        if os.path.isfile(fpath):
            # ディレクトリを設定
            self.filePath = path
            # フルパスを返す
            return fpath
        else:
            return ""
        """
        # <--[inamura 100720]

        """
        # 環境変数PYTHONPATHに指定されたディレクトリを取得
        paths = str( os.environ.get('PYTHONPATH')).split(os.pathsep)
        # 全PYTHONPATHディレクトリを探索
        for path in paths:
            fpath = os.path.normpath(os.path.join(path,fname))
            # ディレクトリに目的ファイルがあるか
            if os.path.isfile(fpath):
                # ディレクトリを設定
                self.filePath = path
                # フルパスを返す
                return fpath

        # 目的ファイルが見つからなかった
        return ""
        """

    ################################
    def CloseDoc(self):
        """
        XMLドキュメントの解放
        @param  無
        @retval 無
        """
        if self.isRead:
            self.xmlDoc.unlink()
            self.isRead = False   # 208/04/28 Minakawa


#######################################
#  PlotMessage
#######################################
class PlotMessage(object):
    """
    各種メッセージの表示クラス
    有効なウィンドウが指定されない場合はコンソールへのテキスト出力
    親ウィンドウが指定されていれば、MessageBoxを表示
    """

    def __init__(self, parent, ex):
        """
        コンストラクタ
        @param parent:親ウィンドウオブジェクト（無ければNoneを指定）
        @param ex:PlotExceptionのインスタンス
        @retval 無し
        """
        msg = ex.GetMessage()

        # MessageBoxを表示する親ウィンドウが無効であればコンソール出力

        if parent is None:
            # traceback.print_stack()
            sys.stdout = codecs.lookup('utf-8')[-1](sys.stdout)
            print(msg)
        else:
            # wx.MessageBox(msg,caption='Message',style=wx.OK)
            ret = QtWidgets.QMessageBox().information(
                None, u"Warning", msg, QtWidgets.QMessageBox.Ok)


#######################################
#  ImagesXpm
#######################################
class ImagesXpm(object):
    """
    Image data for QT
    """
    instance = None

    def __new__(cls, *args):
        """
        Singletonコンストラクタ
        @param  *args 可変引数(無し)
        @retval クラスのインスタンス
        """
        if cls.instance is None:
            # クラスインスタンスが未生成なら新規に構築
            cls.instance = object.__new__(cls, *args)
        return cls.instance

    #########################################
    def GetQPixmap(self, iname):
        if iname in self.imageData:
            return QtGui.QPixmap(self.imageData[iname])
        else:
            return None

    imageData = {
        "Green": ["20 20 3 1 ",
                  "  c black", ". c green", "X c white",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXX   XXXXXXXXXX",
                  "XXXXX  ...  XXXXXXXX",
                  "XXXX ..   .. XXXXXXX",
                  "XXX .  XXX  . XXXXXX",
                  "XXX . XXXXX . XXXXXX",
                  "XX . XXXXXXX . XXXXX",
                  "XX . XXXXXXX . XXXXX",
                  "XXX . XXXXX . XXXXXX",
                  "XXX .  XXX  . XXXXXX",
                  "XXXX ..   .. XXXXXXX",
                  "XXXXX  ...  XXXXXXXX",
                  "XXXXXXX   XXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX",
                  "XXXXXXXXXXXXXXXXXXXX"],
        "Red": ["20 20 3 1 ",
                "  c black", ". c red", "X c white",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXX   XXXXXXXXXX",
                "XXXXX  ...  XXXXXXXX",
                "XXXX ..   .. XXXXXXX",
                "XXX .  XXX  . XXXXXX",
                "XXX . XXXXX . XXXXXX",
                "XX . XXXXXXX . XXXXX",
                "XX . XXXXXXX . XXXXX",
                "XXX . XXXXX . XXXXXX",
                "XXX .  XXX  . XXXXXX",
                "XXXX ..   .. XXXXXXX",
                "XXXXX  ...  XXXXXXXX",
                "XXXXXXX   XXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX",
                "XXXXXXXXXXXXXXXXXXXX"],
        'Processing': ["20 20 4 1 ",
                       "  c black", ". c green", "X c yellow", "o c white",
                       "oooooooooooooooooooo",
                       "oooooooooooooooooooo",
                       "oooooooo ooooooooooo",
                       "oooooooooooooooooooo",
                       "oo oooo   oooo ooooo",
                       "ooooo  .X.  oooooooo",
                       "oooo .X   X. ooooooo",
                       "ooo X  XXX  X oooooo",
                       "ooo . XXXXX . oooooo",
                       "oo X XXXXXXX X o ooo",
                       " o . XXXXXXX . ooooo",
                       "ooo X XXXXX X oooooo",
                       "ooo .  XXX  . oooooo",
                       "oooo X.   .X ooooooo",
                       "ooooo  X.X  oooooooo",
                       "oo oooo   oooo ooooo",
                       "oooooooooooooooooooo",
                       "oooooooo ooooooooooo",
                       "oooooooooooooooooooo",
                       "oooooooooooooooooooo"],
        'GreenEnd': ["20 20 3 1 ",
                     "  c black", ". c green", "X c white",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXX   XXXXXXXXXX",
                     "XXXXX  ...  XXXXXXXX",
                     "XXXX ..   .. XXXXXXX",
                     "XXX .  ...  . XXXXXX",
                     "XXX . ..... . XXXXXX",
                     "XX . ....... . XXXXX",
                     "XX . ....... . XXXXX",
                     "XXX . ..... . XXXXXX",
                     "XXX .  ...  . XXXXXX",
                     "XXXX ..   .. XXXXXXX",
                     "XXXXX  ...  XXXXXXXX",
                     "XXXXXXX   XXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX",
                     "XXXXXXXXXXXXXXXXXXXX"],
        'RedEnd': ["20 20 4 1 ",
                   "  c black", ". c red", "X c green", "o c white",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo",
                   "ooooooo   oooooooooo",
                   "ooooo  XXX  oooooooo",
                   "oooo XX   XX ooooooo",
                   "ooo X  ...  X oooooo",
                   "ooo X ..... X oooooo",
                   "oo X ....... X ooooo",
                   "oo X ....... X ooooo",
                   "ooo X ..... X oooooo",
                   "ooo X  ...  X oooooo",
                   "oooo XX   XX ooooooo",
                   "ooooo  XXX  oooooooo",
                   "ooooooo   oooooooooo",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo",
                   "oooooooooooooooooooo"]
    }


###############################################################################
# テスト用スクリプト
###############################################################################
if __name__ == '__main__':

    pass



# -*- coding: utf-8 -*-

import sys
import os
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + '/Tool')
import ExtMath
import numpy as np
import math

class FreeSliceData:
    def  __init__(self):
         self.visible = True
         self.org = [0.0, 0.0, 0.0]
         self.x   = [0.0, 0.0, 0.0]
         self.y   = [0.0, 0.0, 0.0]
         self.xbin = 0.0
         self.ybin = 0.0
         self.rangex = [0.0, 0.0]
         self.rangey = [0.0, 0.0]
         self.thickness = 0.0
         self.xdiff = 0.0
         self.ydiff = 0.0
         self.zdiff = 1.0

         # test for ***short
         self.org = [-2.0, -2.0, 10.0]
         self.x   = [13.0, -2.0, -5.0]
         self.y   = [13.0, 13.0, 25.0]

         self.xbin = 0.04
         self.ybin = 0.04
         self.rangex = [0, 10]
         self.rangey = [0, 10]
         self.thickness = 1.5

         self.extMath = ExtMath.ExtMath()

    def getNormalVector(self):
        vecX = np.array(self.x) - np.array(self.org)
        vecY = np.array(self.y) - np.array(self.org)

        normal = np.cross(vecX, vecY)
        len = math.sqrt(normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2])
        normal /= len
 
        return normal

    def shiftPlaneAlongNormalVector(self, len, roundFlag, roundPos):
        normal = self.getNormalVector()
        vec = normal * len
        self.shiftPlane(vec[0], vec[1], vec[2], roundFlag, roundPos)

    def shiftPlane(self, xdiff, ydiff, zdiff, roundFlag, roundPos):
        self.org[0] += xdiff
        self.org[1] += ydiff
        self.org[2] += zdiff

        self.x[0] += xdiff
        self.x[1] += ydiff
        self.x[2] += zdiff

        self.y[0] += xdiff
        self.y[1] += ydiff
        self.y[2] += zdiff

        if roundFlag:
            self.org[0] = round(self.org[0], roundPos)  
            self.org[1] = round(self.org[1], roundPos)  
            self.org[2] = round(self.org[2], roundPos)  
            self.x[0] = round(self.x[0], roundPos)  
            self.x[1] = round(self.x[1], roundPos)  
            self.x[2] = round(self.x[2], roundPos)  
            self.y[0] = round(self.y[0], roundPos)  
            self.y[1] = round(self.y[1], roundPos)  
            self.y[2] = round(self.y[2], roundPos)  


    def getPlaneAngle(self):
        vecX = np.array(self.x) - np.array(self.org)
        vecY = np.array(self.y) - np.array(self.org)

        ang = (self.extMath.getAngle(vecX, vecY).item())
        return ang

    def checkPlaneAngle(self):
        vecX = np.array(self.x) - np.array(self.org)
        vecY = np.array(self.y) - np.array(self.org)

        ang = self.getPlaneAngle()
        delta = ang - math.pi/2.0
        eps = 1e-6
        if eps < math.fabs(delta):
            return False
        else:
            return True
        


